/* libiso8601/src/libiso8601/leap.c
 *
 *  (c)2006, Laurence Withers, <l@lwithers.me.uk>.
 *  Released under the GNU GPLv2. See file COPYING or
 *  http://www.gnu.org/copyleft/gpl.html for details.
*/



static int leap_second_days[] = {
    720439, // 19720630
    720623, // 19721231
    720988, // 19731231
    721353, // 19741231
    721718, // 19751231
    722084, // 19761231
    722449, // 19771231
    722814, // 19781231
    723179, // 19791231
    723726, // 19810630
    724091, // 19820630
    724456, // 19830630
    725187, // 19850630
    726101, // 19871231
    726832, // 19891231
    727197, // 19901231
    727744, // 19920630
    728109, // 19930630
    728474, // 19940630
    729023, // 19951231
    729570, // 19970630
    730119, // 19981231
    732676  // 20051231
};



int iso8601_seconds_leap(const struct iso8601_date* date)
{
    int num_ly = 0, i = 0;

    num_ly = sizeof(leap_second_days) / sizeof(int);
    for(i = 0; i < num_ly; ++i) if(leap_second_days[i] == date->day) return 86401;
    return 86400;
}



int iso8601_valid_leap(const struct iso8601_date* date)
{
    int num_ly = 0, i = 0;

    if(date->nsec < 0 || date->nsec >= BILLION) return 0;

    switch(date->sec) {
    case 0 ... 86399:
        return 1;

    case 86400: /* 23:59:60 */
        num_ly = sizeof(leap_second_days) / sizeof(int);
        for(i = 0; i < num_ly; ++i) if(leap_second_days[i] == date->day) return 1;
        return 0;
    }

    return 0;
}



static int _leap_elapsed_day(int sday, int eday)
{
    int spos, epos;

    for(spos = 0; (unsigned)spos < sizeof(leap_second_days) / sizeof(int); ++spos) {
        if(sday <= leap_second_days[spos]) break;
    }
    for(epos = 0; (unsigned)epos < sizeof(leap_second_days) / sizeof(int); ++epos) {
        if(eday <= leap_second_days[epos]) break;
    }

    return spos - epos;
}



int iso8601_leap_elapsed(const struct iso8601_date* start, const struct iso8601_date* end)
{
    return _leap_elapsed_day(start->day, end->day);
}



/* options for text editors
kate: replace-trailing-space-save true; space-indent true; tab-width 4;
vim: expandtab:ts=4:sw=4
*/
